/*
** Module   : DataModelAst.java
** Abstract : A data model-specific AST.
**
** Copyright (c) 2005-2012, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 ECF 20050426   @21000 Created initial version.
** 002 ECF 20050526   @21302 Renamed resolveToLiteral to resolveConstant.
** 003 ECF 20050608   @21461 Removed unused token types.
** 004 ECF 20050616   @21538 Added relation types: MANY_TO_ONE, ONE_TO_ONE
**                           and ONE_TO_MANY.
** 005 ECF 20050628   @21603 Added COMPONENT type.
** 006 ECF 20050707   @21673 Added tokens needed for initial value
**                           processing. INITIAL, STRING, NUM_LITERAL,
**                           DEC_LITERAL, BOOL_TRUE, BOOL_FALSE,
**                           DATE_LITERAL, UNKNOWN.
** 007 ECF 20050720   @21775 Changed COMPONENT to COMPOSITE.
** 008 ECF 20060401   @25332 Added NULL_LITERAL.
** 009 GES 20090518   @42394 Import change.
** 010 OM  20121207          Added support for constructing sequence nodes
**                           into to p2o file.
*/

package com.goldencode.p2j.schema;

import java.util.*;
import antlr.*;
import com.goldencode.ast.*;

/**
 * Provides symbolic token name lookups that are specific to a data model
 * AST. All other function is implemented in the superclass.
 * <p>
 * Currently, line and column number and filename information is not used.
 *
 * @see  DataModelTokenTypes
 */
public class DataModelAst
extends AnnotatedAst
implements DataModelTokenTypes
{
   /** Map of token types to token names */
   private static final Map TOKEN_NAMES = new HashMap();
   
   /** Map of token names to token types */
   private static final Map TOKEN_TYPES = new HashMap();
   
   // Initialize token maps.
   static
   {
      Object[] mapping =
      {
         new Integer(DATA_MODEL)       ,  "DATA_MODEL"   ,
         new Integer(TABLE)            ,  "TABLE"        ,
         new Integer(PROPERTY)         ,  "PROPERTY"     ,
         new Integer(CLASS)            ,  "CLASS"        ,
         new Integer(GENERATOR)        ,  "GENERATOR"    ,
         new Integer(PRIMARY)          ,  "PRIMARY"      ,
         new Integer(INDEX)            ,  "INDEX"        ,
         new Integer(UNIQUE)           ,  "UNIQUE"       ,
         new Integer(INDEX_COL)        ,  "INDEX_COL"    ,
         new Integer(CONSTR_COL)       ,  "CONSTR_COL"   ,
         new Integer(MANY_TO_ONE)      ,  "MANY_TO_ONE"  ,
         new Integer(ONE_TO_MANY)      ,  "ONE_TO_MANY"  ,
         new Integer(ONE_TO_ONE)       ,  "ONE_TO_ONE"   ,
         new Integer(COMPOSITE)        ,  "COMPOSITE"    ,
         new Integer(INITIAL)          ,  "INITIAL"      ,
         new Integer(STRING)           ,  "STRING"       ,
         new Integer(NUM_LITERAL)      ,  "NUM_LITERAL"  ,
         new Integer(DEC_LITERAL)      ,  "DEC_LITERAL"  ,
         new Integer(BOOL_TRUE)        ,  "BOOL_TRUE"    ,
         new Integer(BOOL_FALSE)       ,  "BOOL_FALSE"   ,
         new Integer(DATE_LITERAL)     ,  "DATE_LITERAL" ,
         new Integer(UNKNOWN)          ,  "UNKNOWN"      ,
         new Integer(NULL_LITERAL)     ,  "NULL_LITERAL" ,
         new Integer(SEQUENCE)         ,  "SEQUENCE"     ,
      };
      
      int len = mapping.length;
      for (int i = 0; i < len; i += 2)
      {
         TOKEN_NAMES.put(mapping[i], mapping[i + 1]);
         TOKEN_TYPES.put(mapping[i + 1], mapping[i]);
      }
   }
   
   /**
    * Resolve the given token name to a numeric token type. The lookup is
    * case-insensitive.
    *
    * @param   tokenName
    *          String constant representing token name.
    *
    * @return  Integral token type or <code>null</code> if name could not
    *          be resolved.
    */
   public static Object resolveConstant(String tokenName)
   {
      return TOKEN_TYPES.get(tokenName.toUpperCase());
   }
   
   /**
    * Default constructor.
    */
   public DataModelAst()
   {
      super();
   }
   
   /**
    * Constructor which accepts and wrappers token.
    *
    * @param   token
    *          Token upon which this node is based.
    */
   public DataModelAst(Token token)
   {
      super(token);
   }
   
   /**
    * Translates a text representation of a token type into the
    * actual integer value.
    *
    * @param    tokenName
    *           The text representation of the token type.
    *
    * @return   A valid integer token type value or -1 if no match was found.
    */
   public int lookupTokenType(String tokenName)
   {
      Integer type = (Integer) TOKEN_TYPES.get(tokenName);
      
      return (type != null ? type.intValue() : -1);
   }
   
   /**
    * Translates an integer value of a token type into the associated 
    * human readable text representation.
    *
    * @param    type
    *           The integer token type.
    *
    * @return   A valid symbolic token type name or <code>null</code> if
    *           no match was found.
    */
   public String lookupTokenName(int type)
   {
      return (String) TOKEN_NAMES.get(new Integer(type));
   }
   
   /**
    * Returns the symbolic representation of this AST's token type.
    *
    * @return  Symbolic token name for this AST's token type.
    */
   public String getSymbolicTokenType()
   {
      return lookupTokenName(getType());
   }
}
